VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsDeferral"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 01/2023

Option Explicit

' ----==== Const ====----
Private Const WindowsFoundationDeferral As String = "Windows.Foundation.Deferral"
Private Const IID_IDeferral As String = "{d6269732-3b7f-46a7-b40b-4fdca2a2c693}"
Private Const IID_IDeferralFactory As String = "{65a1ecc5-3fb5-4832-8ca9-f061b281d13a}"

' * Class implements the interface:
' *    Windows.Foundation.IClosable
 
' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IDeferralFactory
    IDeferralFactory_Create = 6
    
    ' IDeferral
    IDeferral_Complete = 6
    
End Enum

' ----==== Variablen ====----
Private m_pIDeferral As Long
Private m_pIDeferralFactory As Long

' ----==== Events ====----
Public Event DeferralCompletedHandler()

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivationFactory(WindowsFoundationDeferral, _
                            IID_IDeferralFactory, _
                            m_pIDeferralFactory) Then
    End If
End Sub

Private Sub Class_Terminate()
    Call Dispose(m_pIDeferral) '<-IClosable
    Call Dispose(m_pIDeferralFactory) '<-IClosable
End Sub

' ----==== Invoke Events ====----
Public Sub Invoke_DeferralCompletedHandler()
    RaiseEvent DeferralCompletedHandler
End Sub

' ----==== Properties ====----
Public Property Get IDeferral() As Long
    IDeferral = m_pIDeferral
End Property

Public Property Let IDeferral(ByVal pIDeferral As Long)
    Call Dispose(m_pIDeferral) '<-IClosable
    m_pIDeferral = IDeferral
End Property

' ----==== Functions ====----
' IDeferralFactory
Public Function Create() As Boolean
    Dim Ret As Boolean
    If m_pIDeferralFactory <> 0& Then
        If Invoke(m_pIDeferralFactory, _
                  IDeferralFactory_Create, _
                  modDeferralCompletedHandler.Create(Me), _
                  VarPtr(m_pIDeferral)) = S_OK Then
            If m_pIDeferral <> 0& Then Ret = True
        End If
    End If
    Create = Ret
End Function
    
' IDeferral
Public Function Complete() As Boolean
    Dim Ret As Boolean
    If m_pIDeferral <> 0& Then
        If Invoke(m_pIDeferral, _
                  IDeferral_Complete) = S_OK Then
            Ret = True
        End If
    End If
    Complete = Ret
End Function

' ----==== Invoke ====----
Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function

