VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsHttpMethod"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 01/2023

Option Explicit

' ----==== Const ====----
Private Const WindowsWebHttpHttpMethod As String = "Windows.Web.Http.HttpMethod"
Private Const IID_IHttpMethod As String = "{728d4022-700d-4fe0-afa5-40299c58dbfd}"
Private Const IID_IHttpMethodStatics As String = "{64d171f0-d99a-4153-8dc6-d68cc4cce317}"
Private Const IID_IHttpMethodFactory As String = "{3c51d10d-36d7-40f8-a86d-e759caf2f83f}"

' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IHttpMethodFactory
    IHttpMethodFactory_Create = 6
    
    ' IHttpMethodStatics
    IHttpMethodStatics_GetDelete = 6
    IHttpMethodStatics_GetGet = 7
    IHttpMethodStatics_GetHead = 8
    IHttpMethodStatics_GetOptions = 9
    IHttpMethodStatics_GetPatch = 10
    IHttpMethodStatics_GetPost = 11
    IHttpMethodStatics_GetPut = 12

    ' IHttpMethod
    IHttpMethod_GetMethod = 6
    
    ' IStringable
    IStringable_ToString = 6
    
End Enum

' ----==== Variablen ====----
Private m_pIStringable As Long
Private m_pIHttpMethod As Long
Private m_pIHttpMethodStatics As Long
Private m_pIHttpMethodFactory As Long

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivationFactory(WindowsWebHttpHttpMethod, _
                            IID_IHttpMethodFactory, _
                            m_pIHttpMethodFactory) Then
        Call QueryIfc(m_pIHttpMethodFactory, IID_IHttpMethodStatics, m_pIHttpMethodStatics)
    End If
End Sub

Private Sub Class_Terminate()
    Call ReleaseIfc(m_pIStringable)
    Call ReleaseIfc(m_pIHttpMethod)
    Call ReleaseIfc(m_pIHttpMethodStatics)
    Call ReleaseIfc(m_pIHttpMethodFactory)
End Sub

' ----==== Properties ====----
Public Property Get IHttpMethod() As Long
    IHttpMethod = m_pIHttpMethod
End Property

Public Property Let IHttpMethod(ByVal pIHttpMethod As Long)
    Call ReleaseIfc(m_pIStringable)
    Call ReleaseIfc(m_pIHttpMethod)
    m_pIHttpMethod = pIHttpMethod
    If m_pIHttpMethod <> 0& Then
        Call QueryIfc(m_pIHttpMethod, IID_IStringable, m_pIStringable)
    End If
End Property

' IHttpMethodStatics
Public Property Get Delete() As clsHttpMethod
    Dim Ret As New clsHttpMethod
    If m_pIHttpMethodStatics <> 0 Then
        Dim pIHttpMethod As Long
        If Invoke(m_pIHttpMethodStatics, _
                  IHttpMethodStatics_GetDelete, _
                  VarPtr(pIHttpMethod)) = S_OK Then
            Ret.IHttpMethod = pIHttpMethod
        End If
    End If
    Set Delete = Ret
End Property

Public Property Get Getting() As clsHttpMethod
    Dim Ret As New clsHttpMethod
    If m_pIHttpMethodStatics <> 0 Then
        Dim pIHttpMethod As Long
        If Invoke(m_pIHttpMethodStatics, _
                  IHttpMethodStatics_GetGet, _
                  VarPtr(pIHttpMethod)) = S_OK Then
            Ret.IHttpMethod = pIHttpMethod
        End If
    End If
    Set Getting = Ret
End Property

Public Property Get Head() As clsHttpMethod
    Dim Ret As New clsHttpMethod
    If m_pIHttpMethodStatics <> 0 Then
        Dim pIHttpMethod As Long
        If Invoke(m_pIHttpMethodStatics, _
                  IHttpMethodStatics_GetHead, _
                  VarPtr(pIHttpMethod)) = S_OK Then
            Ret.IHttpMethod = pIHttpMethod
        End If
    End If
    Set Head = Ret
End Property

Public Property Get Options() As clsHttpMethod
    Dim Ret As New clsHttpMethod
    If m_pIHttpMethodStatics <> 0 Then
        Dim pIHttpMethod As Long
        If Invoke(m_pIHttpMethodStatics, _
                  IHttpMethodStatics_GetOptions, _
                  VarPtr(pIHttpMethod)) = S_OK Then
            Ret.IHttpMethod = pIHttpMethod
        End If
    End If
    Set Options = Ret
End Property

Public Property Get Patch() As clsHttpMethod
    Dim Ret As New clsHttpMethod
    If m_pIHttpMethodStatics <> 0 Then
        Dim pIHttpMethod As Long
        If Invoke(m_pIHttpMethodStatics, _
                  IHttpMethodStatics_GetPatch, _
                  VarPtr(pIHttpMethod)) = S_OK Then
            Ret.IHttpMethod = pIHttpMethod
        End If
    End If
    Set Patch = Ret
End Property

Public Property Get Post() As clsHttpMethod
    Dim Ret As New clsHttpMethod
    If m_pIHttpMethodStatics <> 0 Then
        Dim pIHttpMethod As Long
        If Invoke(m_pIHttpMethodStatics, _
                  IHttpMethodStatics_GetPost, _
                  VarPtr(pIHttpMethod)) = S_OK Then
            Ret.IHttpMethod = pIHttpMethod
        End If
    End If
    Set Post = Ret
End Property

Public Property Get Putting() As clsHttpMethod
    Dim Ret As New clsHttpMethod
    If m_pIHttpMethodStatics <> 0 Then
        Dim pIHttpMethod As Long
        If Invoke(m_pIHttpMethodStatics, _
                  IHttpMethodStatics_GetPut, _
                  VarPtr(pIHttpMethod)) = S_OK Then
            Ret.IHttpMethod = pIHttpMethod
        End If
    End If
    Set Putting = Ret
End Property

' IHttpMethod
Public Property Get Method() As String
    Dim Ret As String
    If m_pIHttpMethod <> 0 Then
        Dim hString As Long
        If Invoke(m_pIHttpMethod, _
                  IHttpMethod_GetMethod, _
                  VarPtr(hString)) = S_OK Then
            Ret = GetWindowsString(hString)
        End If
    End If
    Method = Ret
End Property

' ----==== Functions ====----
Public Function Create(ByVal Method As String) As Boolean
    Dim Ret As Boolean
    If m_pIHttpMethodFactory <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(Method)
        If hString <> 0& Then
            Dim pIHttpMethod As Long
            If Invoke(m_pIHttpMethodFactory, _
                      IHttpMethodFactory_Create, _
                      hString, VarPtr(pIHttpMethod)) = S_OK Then
                If pIHttpMethod <> 0& Then
                    Me.IHttpMethod = pIHttpMethod
                End If
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Create = Ret
End Function

' IStringable
Public Function ToString() As String
    Dim Ret As String
    If m_pIStringable <> 0 Then
        Dim hString As Long
        If Invoke(m_pIStringable, _
                  IStringable_ToString, _
                  VarPtr(hString)) = S_OK Then
            Ret = GetWindowsString(hString)
        End If
    End If
    ToString = Ret
End Function

' ----==== Invoke ====----
Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function

