VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsHttpReqMsg"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 01/2023

Option Explicit

' ----==== Const ====----
Private Const WindowsWebHttpHttpRequestMessage As String = "Windows.Web.Http.HttpRequestMessage"
Private Const IID_IHttpRequestMessage As String = "{f5762b3c-74d4-4811-b5dc-9f8b4e2f9abf}"
Private Const IID_IHttpRequestMessage2 As String = "{c3c60489-62c2-4a3f-9554-226e7c60bd96}"
Private Const IID_IHttpRequestMessageFactory As String = "{5bac994e-3886-412e-aec3-52ec7f25616f}"
Private Const IIterable_IKeyValuePair_HSTRING_IInspectable As String = "{fe2f3d47-5d47-5499-8374-430c7cda0204}"

' * Class implements the interface:
' *    Windows.Foundation.IClosable
 
' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IHttpRequestMessageFactory
    IHttpRequestMessageFactory_Create = 6
    
    ' IHttpRequestMessage
    IHttpRequestMessage_GetContent = 6
    IHttpRequestMessage_PutContent = 7
    IHttpRequestMessage_GetHeaders = 8
    IHttpRequestMessage_GetMethod = 9
    IHttpRequestMessage_PutMethod = 10
    IHttpRequestMessage_GetProperties = 11
    IHttpRequestMessage_GetRequestUri = 12
    IHttpRequestMessage_PutRequestUri = 13
    IHttpRequestMessage_GetTransportInformation = 14

    ' IHttpRequestMessage2
    IHttpRequestMessage2_GetPrivacyAnnotation = 6
    IHttpRequestMessage2_PutPrivacyAnnotation = 7
    
    ' IStringable
    IStringable_ToString = 6
    
    ' IMap_HSTRING_IInspectable
    IMap_HSTRING_IInspectable_GetSize = 7
    
    ' IIterable_IKeyValuePair_HSTRING_IInspectable
    IIterable_IKeyValuePair_HSTRING_IInspectable_First = 6
    
    ' IIterator_IKeyValuePair_HSTRING_IInspectable
    IIterator_IKeyValuePair_HSTRING_IInspectable_GetCurrent = 6
    IIterator_IKeyValuePair_HSTRING_IInspectable_MoveNext = 8

    ' IKeyValuePair_HSTRING_IInspectable
    IKeyValuePair_HSTRING_IInspectable_GetKey = 6
    IKeyValuePair_HSTRING_IInspectable_GetValue = 7

End Enum

' ----==== Variablen ====----
Private m_pIStringable As Long
Private m_pIHttpRequestMessage As Long
Private m_pIHttpRequestMessage2 As Long
Private m_pIHttpRequestMessageFactory As Long

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivationFactory(WindowsWebHttpHttpRequestMessage, _
                            IID_IHttpRequestMessageFactory, _
                            m_pIHttpRequestMessageFactory) Then
    End If
End Sub

Private Sub Class_Terminate()
    Call ReleaseIfc(m_pIStringable)
    Call ReleaseIfc(m_pIHttpRequestMessage2)
    Call Dispose(m_pIHttpRequestMessage) '<-IClosable
    Call Dispose(m_pIHttpRequestMessageFactory) '<-IClosable
End Sub

' ----==== Properties ====----
Public Property Get IHttpRequestMessage() As Long
    IHttpRequestMessage = m_pIHttpRequestMessage
End Property

Public Property Let IHttpRequestMessage(ByVal pIHttpRequestMessage As Long)
    Call ReleaseIfc(m_pIStringable)
    Call ReleaseIfc(m_pIHttpRequestMessage2)
    Call Dispose(m_pIHttpRequestMessage) '<-IClosable
    m_pIHttpRequestMessage = pIHttpRequestMessage
    If m_pIHttpRequestMessage <> 0& Then
        Call QueryIfc(m_pIHttpRequestMessage, IID_IStringable, m_pIStringable)
        Call QueryIfc(m_pIHttpRequestMessage, IID_IHttpRequestMessage2, m_pIHttpRequestMessage2)
    End If
End Property

' IHttpRequestMessage
Public Property Get Content() As clsHttpContent
    Dim Ret As New clsHttpContent
    If m_pIHttpRequestMessage <> 0& Then
        Dim pIHttpContent As Long
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_GetContent, _
                  VarPtr(pIHttpContent)) = S_OK Then
            Ret.IHttpContent = pIHttpContent
        End If
    End If
    Set Content = Ret
End Property

Public Property Let Content(ByVal HttpContent As clsHttpContent)
    If m_pIHttpRequestMessage <> 0& And _
       HttpContent.IHttpContent <> 0& Then
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_PutContent, _
                  HttpContent.IHttpContent) = S_OK Then
        End If
    End If
End Property

Public Property Get Headers() As clsHttpReqHeadCol
    Dim Ret As New clsHttpReqHeadCol
    If m_pIHttpRequestMessage <> 0& Then
        Dim pIHttpRequestHeaderCollection As Long
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_GetHeaders, _
                  VarPtr(pIHttpRequestHeaderCollection)) = S_OK Then
            Ret.IHttpRequestHeaderCollection = pIHttpRequestHeaderCollection
        End If
    End If
    Set Headers = Ret
End Property

Public Property Get Method() As clsHttpMethod
    Dim Ret As New clsHttpMethod
    If m_pIHttpRequestMessage <> 0& Then
        Dim pIHttpMethod As Long
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_GetMethod, _
                  VarPtr(pIHttpMethod)) = S_OK Then
            Ret.IHttpMethod = pIHttpMethod
        End If
    End If
    Set Method = Ret
End Property

Public Property Let Method(ByVal HttpMethod As clsHttpMethod)
    If m_pIHttpRequestMessage <> 0& And _
       HttpMethod.IHttpMethod <> 0& Then
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_PutMethod, _
                  HttpMethod.IHttpMethod) = S_OK Then
        End If
    End If
End Property

Public Property Get PropertiesCount() As Long
    Dim Ret As Long
    If m_pIHttpRequestMessage <> 0& Then
        Dim pIMap_HSTRING_IInspectable As Long
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_GetProperties, _
                  VarPtr(pIMap_HSTRING_IInspectable)) = S_OK Then
            If pIMap_HSTRING_IInspectable <> 0& Then
                Dim value As Long
                If Invoke(pIMap_HSTRING_IInspectable, _
                          IMap_HSTRING_IInspectable_GetSize, _
                          VarPtr(value)) = S_OK Then
                    Ret = value
                End If
                Call ReleaseIfc(pIMap_HSTRING_IInspectable)
            End If
        End If
    End If
    PropertiesCount = Ret
End Property

Friend Property Get Properties() As KeyValuePair()
    Dim Ret() As KeyValuePair
    Dim count As Long
    count = Me.PropertiesCount
    If m_pIHttpRequestMessage <> 0& And count > 0& Then
        ReDim Ret(count - 1)
        Dim pIMap_HSTRING_IInspectable As Long
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_GetProperties, _
                  VarPtr(pIMap_HSTRING_IInspectable)) = S_OK Then
            If pIMap_HSTRING_IInspectable <> 0& Then
                Dim pIIterable_IKeyValuePair_HSTRING_IInspectable As Long
                If QueryIfc(pIMap_HSTRING_IInspectable, _
                            IIterable_IKeyValuePair_HSTRING_IInspectable, _
                            pIIterable_IKeyValuePair_HSTRING_IInspectable) Then
                    Dim pIIterator_IKeyValuePair_HSTRING_IInspectable As Long
                    If Invoke(pIIterable_IKeyValuePair_HSTRING_IInspectable, _
                              IIterable_IKeyValuePair_HSTRING_IInspectable_First, _
                              VarPtr(pIIterator_IKeyValuePair_HSTRING_IInspectable)) = S_OK Then
                        Dim pIKeyValuePair_HSTRING_IInspectable As Long
                        If Invoke(pIIterator_IKeyValuePair_HSTRING_IInspectable, _
                                  IIterator_IKeyValuePair_HSTRING_IInspectable_GetCurrent, _
                                  VarPtr(pIKeyValuePair_HSTRING_IInspectable)) = S_OK Then
                            Dim hString As Long
                            If Invoke(pIKeyValuePair_HSTRING_IInspectable, _
                                      IKeyValuePair_HSTRING_IInspectable_GetKey, _
                                      VarPtr(hString)) = S_OK Then
                                Ret(0).Key = GetWindowsString(hString)
                            End If
                            Dim pIInspectable As Long
                            If Invoke(pIKeyValuePair_HSTRING_IInspectable, _
                                      IKeyValuePair_HSTRING_IInspectable_GetValue, _
                                      VarPtr(pIInspectable)) = S_OK Then
                                Ret(0).value = pIInspectable
                            End If
                            ReleaseIfc (pIKeyValuePair_HSTRING_IInspectable)
                        End If
                        Dim item As Long
                        For item = 1 To count
                            Dim moveNext As Boolean
                            If Invoke(pIIterator_IKeyValuePair_HSTRING_IInspectable, _
                                      IIterator_IKeyValuePair_HSTRING_IInspectable_MoveNext, _
                                      VarPtr(moveNext)) = S_OK Then
                                If moveNext Then
                                    Dim pIKeyValuePair_HSTRING_IInspectable_Next As Long
                                    If Invoke(pIIterator_IKeyValuePair_HSTRING_IInspectable, _
                                              IIterator_IKeyValuePair_HSTRING_IInspectable_GetCurrent, _
                                              VarPtr(pIKeyValuePair_HSTRING_IInspectable_Next)) = S_OK Then
                                        Dim hString_Next As Long
                                        If Invoke(pIKeyValuePair_HSTRING_IInspectable_Next, _
                                                  IKeyValuePair_HSTRING_IInspectable_GetKey, _
                                                  VarPtr(hString_Next)) = S_OK Then
                                            Ret(item).Key = GetWindowsString(hString)
                                        End If
                                        Dim pIInspectable_Next As Long
                                        If Invoke(pIKeyValuePair_HSTRING_IInspectable_Next, _
                                                  IKeyValuePair_HSTRING_IInspectable_GetValue, _
                                                  VarPtr(pIInspectable_Next)) = S_OK Then
                                            Ret(item).value = pIInspectable_Next
                                        End If
                                        Call ReleaseIfc(pIKeyValuePair_HSTRING_IInspectable_Next)
                                    End If
                                End If
                            End If
                        Next
                        Call ReleaseIfc(pIIterator_IKeyValuePair_HSTRING_IInspectable)
                    End If
                    Call ReleaseIfc(pIIterable_IKeyValuePair_HSTRING_IInspectable)
                End If
                Call ReleaseIfc(pIMap_HSTRING_IInspectable)
            End If
        End If
    End If
    Properties = Ret
End Property

Public Property Get RequestUri() As clsURC
    Dim Ret As New clsURC
    If m_pIHttpRequestMessage <> 0& Then
        Dim pIUriRuntimeClass As Long
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_GetRequestUri, _
                  VarPtr(pIUriRuntimeClass)) = S_OK Then
            Ret.IUriRuntimeClass = pIUriRuntimeClass
        End If
    End If
    Set RequestUri = Ret
End Property

Public Property Let RequestUri(ByVal UriRuntimeClass As clsURC)
    If m_pIHttpRequestMessage <> 0& And _
       UriRuntimeClass.IUriRuntimeClass <> 0& Then
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_PutRequestUri, _
                  UriRuntimeClass.IUriRuntimeClass) = S_OK Then
        End If
    End If
End Property

Public Property Get TransportInformation() As clsHttpTransInfo
    Dim Ret As New clsHttpTransInfo
    If m_pIHttpRequestMessage <> 0& Then
        Dim pIHttpTransportInformation As Long
        If Invoke(m_pIHttpRequestMessage, _
                  IHttpRequestMessage_GetTransportInformation, _
                  VarPtr(pIHttpTransportInformation)) = S_OK Then
            Ret.IHttpTransportInformation = pIHttpTransportInformation
        End If
    End If
    Set TransportInformation = Ret
End Property

' IHttpRequestMessage2
Public Property Get PrivacyAnnotation() As String
    Dim Ret As String
    If m_pIHttpRequestMessage2 <> 0& Then
        Dim hString As Long
        If Invoke(m_pIHttpRequestMessage2, _
                  IHttpRequestMessage2_GetPrivacyAnnotation, _
                  VarPtr(hString)) = S_OK Then
            Ret = GetWindowsString(hString)
        End If
    End If
    PrivacyAnnotation = Ret
End Property

Public Property Let PrivacyAnnotation(ByVal value As String)
    If m_pIHttpRequestMessage2 <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(value)
        If hString <> 0& Then
            If Invoke(m_pIHttpRequestMessage2, _
                      IHttpRequestMessage2_PutPrivacyAnnotation, _
                      hString) = S_OK Then
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
End Property

' ----==== Functions ====----
' IHttpRequestMessageFactory
Public Function Create(ByVal HttpMethod As clsHttpMethod, _
                       ByVal UriRuntimeClass As clsURC) As Boolean
    Dim Ret As Boolean
    If m_pIHttpRequestMessageFactory <> 0& And _
       HttpMethod.IHttpMethod <> 0& And _
       UriRuntimeClass.IUriRuntimeClass <> 0& Then
        Dim pIHttpRequestMessage As Long
        If Invoke(m_pIHttpRequestMessageFactory, _
                  IHttpRequestMessageFactory_Create, _
                  HttpMethod.IHttpMethod, _
                  UriRuntimeClass.IUriRuntimeClass, _
                  VarPtr(pIHttpRequestMessage)) = S_OK Then
            If pIHttpRequestMessage <> 0& Then
                Me.IHttpRequestMessage = pIHttpRequestMessage
                Ret = True
            End If
        End If
    End If
    Create = Ret
End Function

' IStringable
Public Function ToString() As String
    Dim Ret As String
    If m_pIStringable <> 0 Then
        Dim hString As Long
        If Invoke(m_pIStringable, _
                  IStringable_ToString, _
                  VarPtr(hString)) = S_OK Then
            Ret = GetWindowsString(hString)
        End If
    End If
    ToString = Ret
End Function

' ----==== Invoke ====----
Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function

