VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsWVCP"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 01/2023

Option Explicit

' ----==== Const ====----
Private Const VT_I8 As Long = 20
 
Private Const WindowsWebUIInteropWebViewControlProcess As String = "Windows.Web.UI.Interop.WebViewControlProcess"
Private Const IID_IWebViewControlProcess As String = "{02c723ec-98d6-424a-b63e-c6136c36a0f2}"
Private Const IID_IWebViewControlProcessFactory As String = "{47b65cf9-a2d2-453c-b097-f6779d4b8e02}"

' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IVectorView
    IVectorView_GetAt = 6
    IVectorView_GetSize = 7
    
    ' IWebViewControlProcessFactory
    IWebViewControlProcessFactory_CreateWithOptions = 6
    
    ' IWebViewControlProcess
    IWebViewControlProcess_GetProcessId = 6
    IWebViewControlProcess_GetEnterpriseId = 7
    IWebViewControlProcess_GetIsPrivateNetworkClientServerCapabilityEnabled = 8
    IWebViewControlProcess_CreateWebViewControlAsync = 9
    IWebViewControlProcess_GetWebViewControls = 10
    IWebViewControlProcess_Terminate = 11
    IWebViewControlProcess_AddProcessExited = 12
    IWebViewControlProcess_RemoveProcessExited = 13

End Enum

' ----==== Oleaut32.dll Declarations ====----
Private Declare Function VariantChangeType Lib "Oleaut32.dll" ( _
                         ByRef pvargDest As Variant, _
                         ByRef pvarSrc As Variant, _
                         ByVal wFlags As Long, _
                         ByVal vt As Long) As Long

' ----==== Variablen ====----
Private m_pIWebViewControlProcess As Long
Private m_pIWebViewControlProcessFactory As Long

Private m_pProcessExited As Long
Private m_pProcessExitedToken As Long

' ----==== Events ====----
Public Event ProcessExited(ByVal sender As Long, ByVal args As Long)

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivationFactory(WindowsWebUIInteropWebViewControlProcess, _
                            IID_IWebViewControlProcessFactory, _
                            m_pIWebViewControlProcessFactory) Then
    End If
End Sub

Private Sub Class_Terminate()
    If m_pProcessExitedToken <> 0& Then
        Call Invoke(m_pIWebViewControlProcess, _
                    IWebViewControlProcess_RemoveProcessExited, _
                    m_pProcessExited, _
                    m_pProcessExitedToken)
    End If
    Call ReleaseIfc(m_pIWebViewControlProcess)
    Call ReleaseIfc(m_pIWebViewControlProcessFactory)
End Sub

' ----==== Invoke Events ====----
Public Sub Invoke_ProcessExited(ByVal sender As Long, ByVal args As Long)
    RaiseEvent ProcessExited(sender, args)
End Sub

' ----==== Properties ====----
Public Property Get IWebViewControlProcess() As Long
    IWebViewControlProcess = m_pIWebViewControlProcess
End Property

Public Property Let IWebViewControlProcess(ByVal pIWebViewControlProcess As Long)
    Call ReleaseIfc(m_pIWebViewControlProcess)
    m_pIWebViewControlProcess = IWebViewControlProcess
End Property

Public Property Get ProcessID() As Long
    Dim Ret As Long
    If m_pIWebViewControlProcess <> 0& Then
        Dim value As Long
        If Invoke(m_pIWebViewControlProcess, _
                  IWebViewControlProcess_GetProcessId, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    ProcessID = Ret
End Property

Public Property Get EnterpriseId() As String
    Dim Ret As String
    If m_pIWebViewControlProcess <> 0& Then
        Dim hString As Long
        If Invoke(m_pIWebViewControlProcess, _
                  IWebViewControlProcess_GetEnterpriseId, _
                  VarPtr(hString)) = S_OK Then
            Ret = GetWindowsString(hString)
        End If
    End If
    EnterpriseId = Ret
End Property

Public Property Get IsPrivateNetworkClientServerCapabilityEnabled() As Boolean
    Dim Ret As Boolean
    If m_pIWebViewControlProcess <> 0& Then
        Dim value As Boolean
        If Invoke(m_pIWebViewControlProcess, _
                  IWebViewControlProcess_GetIsPrivateNetworkClientServerCapabilityEnabled, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    IsPrivateNetworkClientServerCapabilityEnabled = Ret
End Property

' ----==== Functions ====----
Public Function Create() As Boolean
    Dim Ret As Boolean
    If m_pIWebViewControlProcess = 0& Then
        If GetActivateInstance(WindowsWebUIInteropWebViewControlProcess, _
                               IID_IWebViewControlProcess, _
                               m_pIWebViewControlProcess) Then
            If m_pIWebViewControlProcess <> 0& Then
                m_pProcessExited = modITEH_ProcessExited.Create(Me)
                If Invoke(m_pIWebViewControlProcess, _
                          IWebViewControlProcess_AddProcessExited, _
                          m_pProcessExited, _
                          VarPtr(m_pProcessExitedToken)) = S_OK Then
                    Ret = True
                End If
            End If
        End If
    End If
    Create = Ret
End Function

Public Function CreateWithOptions(ByVal WebViewControlProcessOptions As clsWVCPO) As Boolean
    Dim Ret As Boolean
    If m_pIWebViewControlProcessFactory <> 0& And _
       m_pIWebViewControlProcess = 0& And _
       WebViewControlProcessOptions.IWebViewControlProcessOptions <> 0& Then
        Dim pIWebViewControlProcess As Long
        If Invoke(m_pIWebViewControlProcessFactory, _
                  IWebViewControlProcessFactory_CreateWithOptions, _
                  WebViewControlProcessOptions.IWebViewControlProcessOptions, _
                  VarPtr(m_pIWebViewControlProcess)) = S_OK Then
            If m_pIWebViewControlProcess <> 0& Then
                m_pProcessExited = modITEH_ProcessExited.Create(Me)
                If Invoke(m_pIWebViewControlProcess, _
                          IWebViewControlProcess_AddProcessExited, _
                          m_pProcessExited, _
                          VarPtr(m_pProcessExitedToken)) = S_OK Then
                    Ret = True
                End If
            End If
        End If
    End If
    CreateWithOptions = Ret
End Function

Friend Function CreateWebViewControlAsync(ByVal hostWindowHandle As Long, _
                                          ByRef bounds As Rect) As clsWVC
    Dim Ret As New clsWVC
    Dim vHostWindowHandle As Variant
    If VariantChangeType(vHostWindowHandle, hostWindowHandle, 0&, VT_I8) = S_OK Then
        If m_pIWebViewControlProcess <> 0& Then
            Dim pIWebViewControl As Long
            If Invoke(m_pIWebViewControlProcess, _
                      IWebViewControlProcess_CreateWebViewControlAsync, _
                      vHostWindowHandle, _
                      bounds.X, bounds.Y, _
                      bounds.Width, bounds.Height, _
                      VarPtr(pIWebViewControl)) = S_OK Then
                 If Await(pIWebViewControl) Then
                    Ret.IWebViewControl = pIWebViewControl
                 End If
            End If
        End If
    End If
    Set CreateWebViewControlAsync = Ret
End Function

Public Function GetWebViewControlsCount() As Long
    Dim Ret As Long
    If m_pIWebViewControlProcess <> 0& Then
        Dim pIVectorView As Long
        If Invoke(m_pIWebViewControlProcess, _
                  IWebViewControlProcess_GetWebViewControls, _
                  VarPtr(pIVectorView)) = S_OK Then
            Dim count As Long
            If Invoke(pIVectorView, _
                      IVectorView_GetSize, _
                      VarPtr(count)) = S_OK Then
                Ret = count
            End If
            Call ReleaseIfc(pIVectorView)
        End If
    End If
    GetWebViewControlsCount = Ret
End Function

Public Function GetWebViewControls() As clsWVC()
    Dim Ret() As clsWVC
    If m_pIWebViewControlProcess <> 0& Then
        Dim pIVectorView As Long
        If Invoke(m_pIWebViewControlProcess, _
                  IWebViewControlProcess_GetWebViewControls, _
                  VarPtr(pIVectorView)) = S_OK Then
            Dim count As Long
            If Invoke(pIVectorView, _
                      IVectorView_GetSize, _
                      VarPtr(count)) = S_OK Then
                If count > 0& Then
                    Dim item As Long
                    ReDim Ret(count - 1)
                    For item = 0 To count - 1
                        Dim pIWebViewControl As Long
                        If Invoke(pIVectorView, _
                                  IVectorView_GetAt, _
                                  item, _
                                  VarPtr(pIWebViewControl)) = S_OK Then
                            If pIWebViewControl <> 0& Then
                                Set Ret(item) = New clsWVC
                                Ret(item).IWebViewControl = pIWebViewControl
                            End If
                        End If
                    Next
                End If
            End If
            Call ReleaseIfc(pIVectorView)
        End If
    End If
    GetWebViewControls = Ret
End Function

Public Function Terminate() As Boolean
    Dim Ret As Boolean
    If m_pIWebViewControlProcess <> 0& Then
        If Invoke(m_pIWebViewControlProcess, _
                  IWebViewControlProcess_Terminate) = S_OK Then
            Ret = True
        End If
    End If
    Terminate = Ret
End Function

' ----==== Invoke ====----
Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function

